import * as setting from "./setting";
import * as creepApi from "./creepApi"

const createDefaultHarvester = (spawn: StructureSpawn) => {
    return creepApi.SpawnCreep(spawn, setting.mk1Body, { memory: { role: creepApi.RoleToString(setting.Role.Harvester) } });
}
const createDefaultBuilder = (spawn: StructureSpawn) => {
    return creepApi.SpawnCreep(spawn, setting.mk1Body, { memory: { role: creepApi.RoleToString(setting.Role.Builder) } });
};

const createDefaultUpgrader = (spawn: StructureSpawn) => {
    return creepApi.SpawnCreep(spawn, setting.mk1Body, { memory: { role: creepApi.RoleToString(setting.Role.Upgrader) } });
};

const createMk2Harvester = (spawn: StructureSpawn, container: StructureContainer) => {
    return creepApi.SpawnCreep(spawn, setting.Mk2harvesterBody, { memory: { role: creepApi.RoleToString(setting.Role.BigHarvester), workTarget: container.id } });
}

const createMk1Carryer = (spawn: StructureSpawn) => {
    return creepApi.SpawnCreep(spawn, setting.mk1CarryerBody, { memory: { role: creepApi.RoleToString(setting.Role.Carryer) } });
}

export function SpawnCreep(spawn: StructureSpawn) {
    if (spawn.spawning) return;

    let currentCreeps = creepApi.GetCreeps();
    let currentContainer = creepApi.GetContainer(spawn);
    if (currentCreeps.get(creepApi.RoleToString(setting.Role.Harvester)) < setting.HARVESTER_COUNT) {
        if (createDefaultHarvester(spawn) == 0) {
            console.log("Spawn Harvester")
        }
    }
    if (currentCreeps.get(creepApi.RoleToString(setting.Role.Upgrader)) < setting.UPGRADER_COUNT) {
        if (createDefaultUpgrader(spawn) == 0) {
            console.log("Spawn Upgrader")
        }
    }
    if (currentCreeps.get(creepApi.RoleToString(setting.Role.Builder)) < setting.BUILDER_COUNT) {
        if (createDefaultBuilder(spawn) == 0) {
            console.log("Spawn Builder")
        }
    }
    if (currentCreeps.get(creepApi.RoleToString(setting.Role.Carryer)) < setting.CARRYER_COUNT) {
        if (createMk1Carryer(spawn) == 0) {
            console.log("Spawn Carryer")
        }
    }
    if (currentCreeps.get(creepApi.RoleToString(setting.Role.BigHarvester)) < currentContainer.length) {
        for (var structure of Memory.structures) {
            if (structure.structureType == STRUCTURE_CONTAINER && structure.creep == null)
                if (createMk2Harvester(spawn, Game.getObjectById(structure.id) as StructureContainer) == 0) {
                    structure.creep = Game.creeps[spawn.spawning.name].id;
                    console.log("Spawn Mk2Harvester, Set Work Target:", structure.id);
                }
        }
    }
    if (spawn.spawning) {
        var spawningCreep = Game.creeps[spawn.spawning.name];
        spawn.room.visual.text(
            '🛠️' + spawningCreep.memory.role,
            spawn.pos.x + 1,
            spawn.pos.y,
            { align: 'left', opacity: 0.8 });
    }
}